@@ -10,6 +10,7 @@ import subprocess
 import locale
 import codecs
 from functools import wraps
+import unicodedata
 
 import click
 
@@ -26,12 +27,17 @@ from ..inputs import inputs_to_dict
 from ..utils import generate_random_string
 from ..constants import DEFAULT_BLUEPRINT_PATH
 from ..exceptions import SuppressedCloudifyCliError
-from ..exceptions import CloudifyBootstrapError, CloudifyValidationError
+from ..exceptions import (LabelsValidationError,
+                          CloudifyBootstrapError,
+                          CloudifyValidationError,)
 from ..logger import (
     get_logger,
     set_global_verbosity_level,
     DEFAULT_LOG_FILE,
     set_global_json_output)
+from ..filters_utils import (get_filter_rules,
+                             create_labels_filter_rules_list,
+                             create_attributes_filter_rules_list)
 
 
 CLICK_CONTEXT_SETTINGS = dict(
@@ -230,19 +236,7 @@ def parse_and_validate_labels(ctx, param, value):
         raise CloudifyValidationError(
             'ERROR: The `{0}` argument is empty'.format(param.name))
 
-    labels_list = []
-    raw_labels_list = value.split(',')
-    for label in raw_labels_list:
-        if label.count(':') != 1:
-            raise CloudifyValidationError('ERROR: Labels should be of the '
-                                          'form <key>:<value>,<key>:<value>')
-
-        label_key, label_value = label.split(':')
-        validate_param_value('The key of one or more labels', label_key)
-        validate_param_value('The value of one or more labels', label_value)
-        labels_list.append({label_key: label_value})
-
-    return labels_list
+    return get_formatted_labels_list(value)
 
 
 def parse_and_validate_label_to_delete(ctx, param, value):
@@ -253,22 +247,10 @@ def parse_and_validate_label_to_delete(ctx, param, value):
         raise CloudifyValidationError(
             'ERROR: The `{0}` argument is empty'.format(param.name))
 
-    if ',' in value or value.count(':') > 1:
-        raise CloudifyValidationError(
-            'LABEL can be either <key>:<value> or <key>')
-
-    label = value.split(':')
-    if len(label) == 1:
-        validate_param_value('The provided key', label[0])
-        return label[0]
-    else:
-        label_key, label_value = label
-        validate_param_value('The key of the provided label', label_key)
-        validate_param_value('The value of the provided label', label_value)
-        return {label_key: label_value}
+    return get_formatted_labels_list(value, allow_only_key=True)
 
 
-def parse_and_validate_filter_rules(ctx, param, value):
+def validate_value_not_empty(ctx, param, value):
     if value is None or ctx.resilient_parsing:
         return
 
@@ -276,41 +258,71 @@ def parse_and_validate_filter_rules(ctx, param, value):
         raise CloudifyValidationError(
             'ERROR: The `{0}` argument is empty'.format(param.name))
 
-    if any(pattern in value for pattern in
-           ['and', '!=', '=', 'is null', 'is not null']):
-        filter_rules_list = parse_filter_rules_list(value)
-        return {'_filter_rules': filter_rules_list}
-
-    else:
-        validate_param_value('The filter ID', value)
-        return {'_filter_id': value}
+    return value
 
 
-def parse_and_validate_filter_rules_list(ctx, param, value):
-    if value is None or ctx.resilient_parsing:
+def get_formatted_labels_list(raw_labels_string, allow_only_key=False):
+    labels_list = []
+    if any(unicodedata.category(char)[0] == 'C' or char == '"'
+           for char in raw_labels_string):
+        raise CloudifyValidationError(
+            'Error: labels cannot contain control characters or `"`')
+
+    format_err_msg = 'Labels should be of the form <key>:<value>,<key>:<value>'
+    raw_labels_string = raw_labels_string.replace('\\,', '\x00').split(',')
+    for label in raw_labels_string:
+        label = label.replace('\x00', ',')
+        label = label.replace('\\:', '\x00')
+        colons_count = label.count(':')
+        if colons_count == 0:
+            if not allow_only_key:
+                raise LabelsValidationError(label, format_err_msg)
+            label_key, label_value = label, None
+
+        elif colons_count == 1:
+            label_key, label_value = label.split(':')
+            if not label_key or not label_value:
+                raise LabelsValidationError(label, format_err_msg)
+            label_value = label_value.replace('\x00', ':')
+
+        else:
+            if allow_only_key:
+                raise CloudifyValidationError(
+                    'LABEL should be a mixed list of labels and keys. I.e. '
+                    '<key>:<value>,<key>,<key>:<value>')
+            raise LabelsValidationError(label, format_err_msg)
+
+        label_key = label_key.replace('\x00', ':').strip()
+        try:
+            validate_param_value('label_key', label_key)
+        except CloudifyValidationError:
+            raise LabelsValidationError(
+                label, "The label's key contains illegal characters. "
+                       "Only letters, digits and the characters `-`, `.` and "
+                       "`_` are allowed")
+
+        labels_list.append({label_key: label_value})
+
+    return labels_list
+
+
+def _validate_filter_rules_not_empty(ctx, param, value):
+    if value is None or value == () or ctx.resilient_parsing:
         return
 
     if not value:
         raise CloudifyValidationError(
             'ERROR: The `{0}` argument is empty'.format(param.name))
 
-    return parse_filter_rules_list(value)
+
+def parse_labels_filter_rules(ctx, param, value):
+    _validate_filter_rules_not_empty(ctx, param, value)
+    return create_labels_filter_rules_list(value)
 
 
-def parse_filter_rules_list(raw_filter_rules):
-    filter_rules = []
-    for filter_rule in raw_filter_rules.split('and'):
-        if all(pattern not in filter_rule for pattern in
-               ['=', 'is null', 'is not null']):
-            raise CloudifyValidationError(
-                'ERROR: Filter rules must be one of: <key>=<value>, '
-                '<key>=[<value1>,<value2>,...], <key>!=<value>, '
-                '<key>!=[<value1>,<value2>,...], <key> is null, '
-                '<key> is not null')
-        filter_rules.append(filter_rule.strip().lower())
-        # The rest of the validation is done in the rest-service
-
-    return filter_rules
+def parse_attributes_filter_rules(ctx, param, value):
+    _validate_filter_rules_not_empty(ctx, param, value)
+    return create_attributes_filter_rules_list(value)
 
 
 def validate_name(ctx, param, value):
@@ -651,6 +663,10 @@ class AliasedGroup(CommandMixin, click.Group):
         kw.setdefault('cls', CommandWithLoggers)
         return super(AliasedGroup, self).command(*a, **kw)
 
+    def group(self, *a, **kw):
+        kw.setdefault('cls', self.__class__)
+        return super(AliasedGroup, self).group(*a, **kw)
+
 
 def group(name):
     """Allow to create a group with a default click context
@@ -862,6 +878,11 @@ class Options(object):
             is_flag=True,
             help=helptexts.SKIP_REINSTALL)
 
+        self.dont_skip_reinstall = click.option(
+            '--dont-skip-reinstall',
+            is_flag=True,
+            help=helptexts.DONT_SKIP_REINSTALL)
+
         self.ignore_failure = click.option(
             '--ignore-failure',
             is_flag=True,
@@ -1046,11 +1067,6 @@ class Options(object):
             required=False,
             help=helptexts.NODE_NAME)
 
-        self.without_deployment_envs = click.option(
-            '--without-deployment-envs',
-            is_flag=True,
-            help=helptexts.RESTORE_SNAPSHOT_EXCLUDE_EXISTING_DEPLOYMENTS)
-
         self.exclude_credentials = click.option(
             '--exclude-credentials',
             is_flag=True,
@@ -1389,6 +1405,14 @@ class Options(object):
             help=helptexts.WITH_LOGS
         )
 
+        self.delete_deployments = click.option(
+            '--delete-deployments',
+            required=False,
+            is_flag=True,
+            default=False,
+            help=helptexts.DELETE_GROUP_DEPLOYMENTS
+        )
+
         self.port = click.option(
             '--port',
             required=False,
@@ -1527,8 +1551,7 @@ class Options(object):
             callback=set_manager)
 
         self.all_blueprints = click.option(
-            '-a',
-            '--all',
+            '--all-blueprints',
             'all_blueprints',
             is_flag=True,
             default=False,
@@ -1582,7 +1605,18 @@ class Options(object):
             callback=parse_and_validate_labels,
             help=helptexts.LABELS
         )
-
+        self.deployment_labels = click.option(
+            '--deployment-labels',
+            required=False,
+            callback=parse_and_validate_labels,
+            help=helptexts.LABELS
+        )
+        self.blueprint_labels = click.option(
+            '--blueprint-labels',
+            required=False,
+            callback=parse_and_validate_labels,
+            help=helptexts.LABELS
+        )
         self.async_upload = click.option(
             '-a',
             '--async-upload',
@@ -1591,24 +1625,25 @@ class Options(object):
             help=helptexts.ASYNC_UPLOAD,
         )
 
-        self.update_filter_rules = click.option(
-            '--filter-rules',
-            callback=parse_and_validate_filter_rules_list,
-            help=helptexts.FILTER_RULES
-        )
+        self.schedule_name = click.option(
+            '-n',
+            '--schedule-name',
+            required=False,
+            help=helptexts.SCHEDULE_NAME)
 
-        self.filter_rules = click.option(
-            '--filter',
-            callback=parse_and_validate_filter_rules,
-            help=helptexts.FILTER_RULES_OR_ID
-        )
-
-        self.frequency = click.option(
-            '--freq',
+        self.recurrence = click.option(
+            '-r',
+            '--recurrence',
             cls=MutuallyExclusiveOption,
             mutually_exclusive=['rrule'],
             required=False,
-            help=helptexts.SCHEDULE_FREQUENCY
+            help=helptexts.SCHEDULE_RECURRENCE
+        )
+
+        self.tz = click.option(
+            '--tz',
+            required=False,
+            help=helptexts.TIMEZONE
         )
 
         self.count = click.option(
@@ -1626,13 +1661,15 @@ class Options(object):
             cls=MutuallyExclusiveOption,
             mutually_exclusive=['rrule'],
             required=False,
-            help=helptexts.SCHEDULE_WEEKDAYS
+            help=helptexts.SCHEDULE_WEEKDAYS,
+            multiple=True,
+            callback=self.parse_comma_separated,
         )
 
         self.rrule = click.option(
             '--rrule',
             cls=MutuallyExclusiveOption,
-            mutually_exclusive=['frequency', 'count', 'weekdays'],
+            mutually_exclusive=['recurrence', 'count', 'weekdays'],
             required=False,
             help=helptexts.SCHEDULE_RRULE
         )
@@ -1675,6 +1712,59 @@ class Options(object):
             type=int
         )
 
+        self.deployment_group_filter_id = click.option(
+            '--filter-id',
+            help=helptexts.DEP_GROUP_FILTER_ID
+        )
+
+        self.deployment_group_deployments_from_group = click.option(
+            '--from-group',
+            help=helptexts.DEP_GROUP_FROM_GROUP,
+        )
+
+        self.into_environments_group = click.option(
+            '--into-environments', 'environments_group',
+            help=helptexts.DEP_GROUP_INTO_ENVIRONMENTS,
+            cls=MutuallyExclusiveOption,
+            mutually_exclusive=['count'],
+        )
+
+        self.deployment_group_id = click.option(
+            '-g', '--deployment-group-id',
+            help=helptexts.DEP_GROUP_ID,
+        )
+
+        self.group_id_filter = click.option(
+            '--group-id',
+            help=helptexts.GROUP_ID_FILTER,
+        )
+
+        self.filter_id = click.option(
+            '--filter-id',
+            callback=validate_name,
+            help=helptexts.FILTER_ID
+        )
+
+        self.generate_id = click.option(
+            '--generate-id',
+            is_flag=True,
+            default=False,
+            help=helptexts.GENERATE_ID
+        )
+
+        self.display_name = click.option(
+            '-n',
+            '--display-name',
+            callback=validate_value_not_empty,
+            help=helptexts.DISPLAY_NAME
+        )
+
+        self.search_name = click.option(
+            '--search-name',
+            callback=validate_value_not_empty,
+            help=helptexts.SEARCH_NAME
+        )
+
     def common_options(self, f):
         """A shorthand for applying commonly used arguments.
 
@@ -1919,13 +2009,15 @@ class Options(object):
             callback=_get_validate_callback(validate))
 
     @staticmethod
-    def blueprint_path(required=False):
+    def blueprint_path(required=False,
+                       extra_message='',
+                       exists=True):
         return click.option(
             '-p',
             '--blueprint-path',
             required=required,
-            type=click.Path(exists=True),
-            help=helptexts.BLUEPRINT_PATH)
+            type=click.Path(exists=exists),
+            help=helptexts.BLUEPRINT_PATH + extra_message)
 
     @staticmethod
     def tenant_role(help_text, required, options_flags=None):
@@ -2068,6 +2160,34 @@ class Options(object):
             kwargs['mutually_exclusive'] = mutually_exclusive_with
         return click.option('to_datetime', '--to', **kwargs)
 
+    @staticmethod
+    def since(required=False, mutually_exclusive_with=None, help_lead=None):
+        help_lead = help_lead or 'The earliest possible time to run'
+        kwargs = {
+            'required': required,
+            'type': str,
+            'expose_value': True,
+            'help': helptexts.TIME_EXPRESSION.format(help_lead),
+        }
+        if mutually_exclusive_with:
+            kwargs['cls'] = MutuallyExclusiveOption
+            kwargs['mutually_exclusive'] = mutually_exclusive_with
+        return click.option('-s', '--since', **kwargs)
+
+    @staticmethod
+    def until(required=False, mutually_exclusive_with=None, help_lead=None):
+        help_lead = help_lead or 'The latest possible time to run'
+        kwargs = {
+            'required': required,
+            'type': str,
+            'expose_value': True,
+            'help': helptexts.TIME_EXPRESSION.format(help_lead),
+        }
+        if mutually_exclusive_with:
+            kwargs['cls'] = MutuallyExclusiveOption
+            kwargs['mutually_exclusive'] = mutually_exclusive_with
+        return click.option('-u', '--until', **kwargs)
+
     @staticmethod
     def before(required=False,
                mutually_exclusive_with=None,
@@ -2138,5 +2258,48 @@ class Options(object):
             required=False,
             help=help)
 
+    @staticmethod
+    def _filter_rules(f, resource):
+        help_text = (helptexts.DEPLOYMENTS_ATTRS_FILTER_RULES if
+                     resource == 'deployment' else
+                     helptexts.BLUEPRINTS_ATTRS_FILTER_RULES)
+        attrs_rule = click.option(
+            '-ar',
+            '--attrs-rule',
+            callback=parse_attributes_filter_rules,
+            help=help_text,
+            multiple=True
+        )
+
+        labels_rule = click.option(
+            '-lr',
+            '--labels-rule',
+            callback=parse_labels_filter_rules,
+            help=helptexts.LABELS_FILTER_RULES,
+            multiple=True
+        )
+
+        def _filter_rules_deco(f):
+            @wraps(f)
+            def _inner(*args, **kwargs):
+                filter_rules = get_filter_rules(
+                    kwargs.pop('labels_rule', None),
+                    kwargs.pop('attrs_rule', None))
+
+                kwargs['filter_rules'] = filter_rules
+                return f(*args, **kwargs)
+            return _inner
+
+        for arg in [attrs_rule, labels_rule, _filter_rules_deco]:
+            f = arg(f)
+
+        return f
+
+    def blueprint_filter_rules(self, f):
+        return self._filter_rules(f, 'blueprint')
+
+    def deployment_filter_rules(self, f):
+        return self._filter_rules(f, 'deployment')
+
 
 options = Options()
