@@ -24,13 +24,14 @@ import logging
 import logging.config
 
 import colorama
+from contextlib import contextmanager
 
 from cloudify import logs
 
 from . import env
 from .config.config import is_use_colors
 from .config.config import CloudifyConfig
-from .colorful_event import ColorfulEvent
+from .colorful_event import ColorfulEvent, ColorfulGroupEvent
 
 DEFAULT_LOG_FILE = os.path.join(env.CLOUDIFY_WORKDIR, 'logs', 'cli.log')
 
@@ -136,7 +137,7 @@ def _configure_defaults(logger_config):
     logger_config['handlers']['file']['filename'] = DEFAULT_LOG_FILE
     logfile_dir = os.path.dirname(DEFAULT_LOG_FILE)
     if not os.path.exists(logfile_dir):
-        os.makedirs(logfile_dir)
+        os.makedirs(logfile_dir, mode=0o700)
 
 
 def _configure_from_file(loggers_config):
@@ -147,7 +148,7 @@ def _configure_from_file(loggers_config):
     loggers_config['handlers']['file']['filename'] = config.logging.filename
     logfile_dir = os.path.dirname(config.logging.filename)
     if not os.path.exists(logfile_dir):
-        os.makedirs(logfile_dir)
+        os.makedirs(logfile_dir, mode=0o700)
 
     # add handlers to every logger specified in the file
     for logger_name, logging_level in config.logging.loggers.items():
@@ -177,7 +178,11 @@ def get_events_logger(json_output):
         :return:
         """
         for event in events:
-            output = logs.create_event_message_prefix(event)
+            event_class = None
+            if event.get('execution_group_id') is not None:
+                event_class = ColorfulGroupEvent
+            with _nest_event_class(event_class):
+                output = logs.create_event_message_prefix(event)
             if output:
                 click.echo(output)
 
@@ -217,3 +222,12 @@ class CloudifyJSONEncoder(json.JSONEncoder):
         if isinstance(obj, uuid.UUID):
             return obj.hex
         return super(CloudifyJSONEncoder, self).default(obj)
+
+
+@contextmanager
+def _nest_event_class(event_class):
+    prev_event_class = logs.EVENT_CLASS
+    if event_class:
+        logs.EVENT_CLASS = event_class
+    yield
+    logs.EVENT_CLASS = prev_event_class
