@@ -148,11 +148,14 @@ def print_details(data, title):
         output(json.dumps(data, cls=CloudifyJSONEncoder))
     else:
         output(title)
-        for item in data.items():
-            field_name = str(item[0]) + ':'
-            field_value = str(item[1])
-            field_value = get_timestamp(field_value) or field_value
-            output('\t{0} {1}'.format(field_name.ljust(16), field_value))
+        if data and isinstance(data, dict):
+            max_name_width = max(len(k) for k in data.keys()) + 5
+            for item in data.items():
+                field_name = str(item[0]) + ':'
+                field_value = str(item[1])
+                field_value = get_timestamp(field_value) or field_value
+                output('\t{0} {1}'.format(field_name.ljust(max_name_width),
+                                          field_value))
 
 
 def get_timestamp(data):
